/*
 *
 * Amiga Smart File System, Linux implementation
 * version: 1.0beta7
 *  
 * Copyright (C) 2003,2004  Marek 'March' Szyprowski <marek@amiga.pl>
 *
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 */

#include <linux/types.h>
#include <linux/errno.h>
#include <linux/slab.h>
#include <linux/fs.h>
#include <linux/buffer_head.h>
#include <linux/vfs.h>
#include <linux/pagemap.h>
#include "asfs_fs.h"

#include <asm/byteorder.h>
#include <asm/uaccess.h>

int asfs_symlink_readpage(struct file *file, struct page *page)
{
	struct buffer_head *bh;
	struct fsSoftLink *slinkcont;
	struct inode *inode = page->mapping->host;
	struct super_block *sb = inode->i_sb;
	char *link = kmap(page);
	int i = 0, j = 0;
	char c, lc = 0, *prefix, *lf, *p;

	if (!(bh = asfs_breadcheck(sb, ASFS_I(inode)->firstblock, ASFS_SOFTLINK_ID))) {
		SetPageError(page);
		kunmap(page);
		unlock_page(page);
		return -EIO;
	}
	slinkcont = (struct fsSoftLink *) bh->b_data;

	lf = slinkcont->string;
	prefix = ASFS_SB(sb)->prefix ? ASFS_SB(sb)->prefix : "/";

	if ((p = strchr(lf,':'))) {	/* Handle assign or volume name */
		if (ASFS_SB(sb)->root_volume &&
		    strncmp(lf, ASFS_SB(sb)->root_volume, strlen(ASFS_SB(sb)->root_volume)) == 0) {
			/* global root volume name found */
			link[i++] = '/';
			lf = p+1;
		} else {
			/* adding volume prefix */ 
			while (i < 1023 && (c = prefix[i]))
				link[i++] = c;
			if (ASFS_SB(sb)->flags & ASFS_VOL_LOWERCASE) {
				while (i < 1023 && lf[j] != ':')
					link[i++] = asfs_lowerchar(lf[j++]);
			} else {
				while (i < 1023 && lf[j] != ':')
					link[i++] = lf[j++];
			}
			if (i < 1023)
				link[i++] = '/';
			j++;
		}
		lc = '/';
	}

	while (i < 1023 && (c = lf[j])) {
		if (c == '/' && lc == '/' && i < 1020) {	/* parent dir */
			link[i++] = '.';
			link[i++] = '.';
		}
		link[i++] = c;
		lc = c;
		j++;
	}
	link[i] = '\0';
	SetPageUptodate(page);
	kunmap(page);
	unlock_page(page);
	asfs_brelse(bh);
	return 0;
}

#ifdef CONFIG_ASFS_RW

int asfs_write_symlink(struct inode *symfile, const char *symname)
{
	struct super_block *sb = symfile->i_sb;
	struct buffer_head *bh;
	struct fsSoftLink *slinkcont;
	char *p, c, lc;
	int i, maxlen, pflen;

	asfs_debug("asfs_write_symlink %s to node %d\n", symname, (int)symfile->i_ino);

	if (!(bh = asfs_breadcheck(sb, ASFS_I(symfile)->firstblock, ASFS_SOFTLINK_ID))) {
		unlock_super(sb);
		return -EIO;
	}
	slinkcont = (struct fsSoftLink *) bh->b_data;

	/* translating symlink target path */

	maxlen = sb->s_blocksize - sizeof(struct fsSoftLink) - 2;
	i  = 0;
	p  = slinkcont->string;
	lc = '/';

	if (*symname == '/') {
		while (*symname == '/')
			symname++;
		if (ASFS_SB(sb)->prefix &&
		    strncmp(symname-1, ASFS_SB(sb)->prefix, (pflen = strlen(ASFS_SB(sb)->prefix))) == 0) {
			/* found volume prefix, ommiting it */
			symname += pflen;
			while ((c = *symname++) != '/' && c != '\0') {
				*p++ = c;
				i++;
			}
			*p++ = ':';
		} else if (ASFS_SB(sb)->root_volume) {	/* adding root volume name */
			while (ASFS_SB(sb)->root_volume[i])
				*p++ = ASFS_SB(sb)->root_volume[i++];
			*p++ = ':';
		} else {	/* do nothing */
			*p++ = '/';
		}
		i++;
	}

	while (i < maxlen && (c = *symname++)) {
		if (c == '.' && lc == '/' && *symname == '.' && symname[1] == '/') {
			*p++ = '/';
			i++;
			symname += 2;
			lc = '/';
		} else if (c == '.' && lc == '/' && *symname == '/') {
			symname++;
			lc = '/';
		} else {
			*p++ = c;
			lc   = c;
			i++;
		}
		if (lc == '/')
			while (*symname == '/')
				symname++;
	}
	*p = 0;

	asfs_bstore(sb, bh);
	asfs_brelse(bh);

	unlock_super(sb);
	
	return 0;
}

#endif
