/* trace - display the events traced by Bossa
 *
 * (c) Christophe Augier
 *
 * output format is:
 *    sec.usec:event_type:pid_of_tgt:pid_of_current
 *
 * change the value of BEDTIME to change the frequency
 * the data are read from the kernel.
 * 
 * (3s. default, the size of the buffer in the kernel 
 * is fixed to 3000 events, so you may need to tune
 * BEDTIME).
 */

#include <stdlib.h>
/* open, ioctl */
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
/* printf */
#include <stdio.h>
/* sleep */
#include <unistd.h>
/* signal */
#include <signal.h>

#define BEDTIME 3

struct trace_struct {
	struct timeval time;
	//unsigned long long int time;
	int type;
	int target;
	int current;
} t_buffer[3000];

#define IOCTL_TOGGLE_TRACE 111

	int fd;

void cleanup(int s) {
	int res;
	int i;
/* toggle bossa trace support off */
	res = ioctl(fd,IOCTL_TOGGLE_TRACE, &i);
	if (!res) fprintf(stderr, "Ending the trace.\n");

	exit(0);
}
int main(void) {

	int res;
	int i;

/* open bossa device */
fprintf(stderr, "Opening bossa device\n");
	fd = open ("/dev/misc/bossa", O_RDWR);
	if (fd < 0) perror("trace, open");

	signal(SIGINT, cleanup);
/* toggle bossa trace support on */
//printf("Ioctling bossa device\n");
	res = ioctl(fd,IOCTL_TOGGLE_TRACE, &i);
	if (!res) fprintf(stderr, "Beginning the trace.\n");

/* execute script file */
	//TODO

/* read and display bossa traces */
	while (1) {
		res = read(fd, t_buffer, 1000);
		if (res == 0) {
			fprintf(stderr, "Sleeping ...\n");
			sleep(BEDTIME);
		}
		for(i = 0; i < res; i++) {
			printf("%d.%0.6d:%x:%d:%d\n",  t_buffer[i].time.tv_sec,
					t_buffer[i].time.tv_usec, t_buffer[i].type,
					t_buffer[i].target, t_buffer[i].current);
		}
	}

	cleanup(0);
return 0;

}

